unit KA.Data.KADao.DBColumnCheckListBox;
interface

uses
  Borland.Vcl.SysUtils,
  Borland.Vcl.Classes,
  Borland.Vcl.DB,
  Borland.Vcl.DBCtrls,
  Borland.Vcl.CheckLst,
  Borland.Vcl.Dialogs;

type
  AddItemEvent = Procedure(Sender : TObject; Index : Integer; Var ItemText:String; Bookmark:String; Var CanDo : Boolean) of Object;

  TKADaoDBColumnCheckListBox = class(TCheckListBox)
  private
    { Private declarations }
    FDataLink       : TFieldDataLink;
    FMasterLink     : TFieldDataLink;
    FInGetting      : Boolean;
    FUnique         : Boolean;
    FGetData        : Boolean;
    FAutoPost       : Boolean;
    FReadOnly       : Boolean;
    FBookmarks      : TStringList;
    FOnBeforeAddItem: AddItemEvent;
    FOnAfterAddItem : AddItemEvent;

    Procedure          DataChange(Sender: TObject);
    Procedure          UpdateData(Sender: TObject);
    Procedure          EditingChange(Sender: TObject);
    Procedure          ActiveChange(Sender: TObject);

    Procedure          ML_DataChange(Sender: TObject);
    Procedure          ML_UpdateData(Sender: TObject);
    Procedure          ML_EditingChange(Sender: TObject);
    Procedure          ML_ActiveChange(Sender: TObject);

    Procedure          FillData;
  protected
    { Protected declarations }
    Function    GetDataField: string;
    Procedure   SetDataField(const Value: string);
    Function    GetDataSource: TDataSource;
    Procedure   SetDataSource(Value: TDataSource);
    Function    GetMasterSource: TDataSource;
    Procedure   SetMasterSource(Value: TDataSource);
    Procedure   SetUnique(Value: Boolean);
    Procedure   SetBookmarks(Value : TStringList);
    Function    IndexOf(Bookmark:String):Integer;
    Procedure   Notification(AComponent: TComponent; Operation: TOperation); override;
    Procedure   Click; override;
    Procedure   Loaded; override;
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent);override;
    Destructor  Destroy;override;
    Procedure   Resync;
    Property    Bookmarks        : TStringList  Read FBookmarks        Write SetBookmarks;
  published
    { Published declarations }
    Property    AutoPost          : Boolean      Read FAutoPost         Write FAutoPost;
    Property    DataField         : String       Read GetDataField    Write SetDataField;
    Property    DataSource        : TDataSource  Read GetDataSource   Write SetDataSource;
    Property    MasterSource      : TDataSource  Read GetMasterSource Write SetMasterSource;
    Property    ReadOnly          : Boolean      Read FReadOnly         Write FReadOnly;
    Property    UniqueDataOnly    : Boolean      Read FUnique           Write SetUnique;
    Property    OnBeforeAddItem   : AddItemEvent Read FOnBeforeAddItem  Write FOnBeforeAddItem;
    Property    OnAfterAddItem    : AddItemEvent Read FOnAfterAddItem   Write FOnAfterAddItem;
  end;

procedure Register;

implementation

Constructor TKADaoDBColumnCheckListBox.Create(AOwner: TComponent);
Begin
  inherited Create(AOwner);
  FAutoPost                   := False;
  FReadOnly                   := True;
  FInGetting                  := False;
  FGetData                    := False;
  FUnique                     := False;
  FBookmarks                  := TStringList.Create;

  FMasterLink                 := TFieldDataLink.Create;
  FMasterLink.Control         := Self;
  FMasterLink.OnDataChange    := ML_DataChange;
  FMasterLink.OnEditingChange := ML_EditingChange;
  FMasterLink.OnUpdateData    := ML_UpdateData;
  FMasterLink.OnActiveChange  := ML_ActiveChange;

  FDataLink                   := TFieldDataLink.Create;
  FDataLink.Control           := Self;
  FDataLink.OnDataChange      := DataChange;
  FDataLink.OnEditingChange   := EditingChange;
  FDataLink.OnUpdateData      := UpdateData;
  FDataLink.OnActiveChange    := ActiveChange;
End;

Destructor TKADaoDBColumnCheckListBox.Destroy;
Begin
 FBookmarks.Free;
 FMasterLink.Free;
 FDataLink.Free;
 FDataLink := nil;
 inherited Destroy;
End;

Function  TKADaoDBColumnCheckListBox.IndexOf(Bookmark:String):Integer;
Var
  X : Integer;
Begin
  For X := 0 To FBookmarks.Count-1 do
      Begin
        Result := X;
        If Bookmark=FBookmarks.Strings[X] Then Exit;
      End;
  Result := -1;
End;

procedure TKADaoDBColumnCheckListBox.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and (AComponent = DataSource) then
      Begin
        DataSource := nil;
      End;
  if (Operation = opRemove) and (FMasterLink <> nil) and (AComponent = MasterSource) then
      Begin
        MasterSource := nil;
      End;
end;


Procedure TKADaoDBColumnCheckListBox.Click;
Begin
  Inherited Click;
  If  (Assigned(FDataLink.DataSource))
  And (FDataLink.Active)
  And (FDataLink.FieldName<> '')
  And (NOT FInGetting)
  And (Assigned(FDataLink.Field))
  And (NOT FDataLink.Field.IsBlob)
  And (NOT FUNIQUE)
  And (FReadOnly) Then
      Begin
        Try
          if ItemIndex > -1 Then FDataLink.Dataset.Bookmark := FBookMarks.Strings[ItemIndex];
        Except
        End;
      End;
  //****************************************************************************
  If  (Assigned(FDataLink.DataSource))
  And (FDataLink.Active)
  And (FDataLink.FieldName<> '')
  And (NOT FInGetting)
  And (Assigned(FDataLink.Field))
  And (NOT FDataLink.Field.IsBlob)
  And (NOT FReadOnly)
  And (FDataLink.CanModify)
  And (FDataLink.Edit)  Then
      Begin
       FDataLink.Modified;
       FDataLink.UpdateRecord;
       if FAutoPost Then FDataLink.DataSet.Post;
      End;
End;


Procedure TKADaoDBColumnCheckListBox.Loaded;
Begin
 Inherited Loaded;
 Items.Clear;
 FBookmarks.Clear;
End;

Procedure TKADaoDBColumnCheckListBox.FillData;
Var
  BK     : TBookmarkStr;
  S      : String;
  CanDo  : Boolean;
Begin
  Items.BeginUpdate;
  Try
    Items.Clear;
    FBookmarks.Clear;
    If  (Assigned(FDataLink.DataSource))
    And (FDataLink.Active)
    And (FDataLink.FieldName<> '')
    And (FDataLink.DataSet.State=dsBrowse)
    And (NOT FDataLink.DataSet.IsEmpty)
    And (NOT FInGetting)
    And (Assigned(FDataLink.Field))
    And (NOT FDataLink.Field.IsBlob) Then
      Begin
        FInGetting := True;
        BK := FDataLink.DataSet.Bookmark;
        FDataLink.DataSet.DisableControls;
        Try
         FDataLink.DataSet.First;
         While NOT FDataLink.DataSet.EOF do
           Begin
             S := FDataLink.DataSet.FieldByName(FDataLink.FieldName).AsString;
             if FUnique Then
                Begin
                  if Items.IndexOf(S) = -1 Then
                     Begin
                       CanDo := True;
                       if Assigned(FOnBeforeAddItem) Then FOnBeforeAddItem(Self,Items.Count,S,FDataLink.DataSet.Bookmark,CanDo);
                       if CanDo Then
                          Begin
                            Items.Add(S);
                            FBookmarks.Add(FDataLink.DataSet.Bookmark);
                            CanDo := True;
                            if Assigned(FOnAfterAddItem) Then FOnAfterAddItem(Self,Items.Count-1,S,FDataLink.DataSet.Bookmark, CanDo);
                          End;
                     End;
                End
             Else
                Begin
                  CanDo := True;
                  if Assigned(FOnBeforeAddItem) Then FOnBeforeAddItem(Self,Items.Count,S,FDataLink.DataSet.Bookmark,CanDo);
                  if CanDo Then
                     Begin
                       Items.Add(S);
                       FBookmarks.Add(FDataLink.DataSet.Bookmark);
                       CanDo := True;
                       if Assigned(FOnAfterAddItem) Then FOnAfterAddItem(Self,Items.Count-1,S,FDataLink.DataSet.Bookmark,CanDo);
                     End;
                End;
             FDataLink.DataSet.Next;
           End;
        Finally
          Try
            FDataLink.DataSet.Bookmark := BK;
          Except
          End;  
          ItemIndex := IndexOf(BK);
          FDataLink.DataSet.EnableControls;
          FInGetting := False;
        End;
      End;
  Finally
    Items.EndUpdate;
  End;
End;

Procedure TKADaoDBColumnCheckListBox.ActiveChange(Sender: TObject);
Begin
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet.Active) Then
      Begin
        Enabled := True;
        FillData;
      End
  Else
      Begin
        Items.Clear;
        FBookmarks.Clear;
        Enabled := False;
      End;
End;

Procedure TKADaoDBColumnCheckListBox.Resync;
Var
 S  : String;
Begin
 If  (Assigned(FDataLink.DataSource))
  And (FDataLink.Active)
  And (FDataLink.FieldName<> '')
  And (NOT FInGetting)
  And (NOT MultiSelect)
  And (Assigned(FDataLink.Field))
  And (NOT FDataLink.Field.IsBlob)
  And (NOT FDataLink.DataSet.ControlsDisabled)
  And (FDataLink.Dataset.State = dsBrowse) Then
      Begin
        if FUnique Then
            Begin
             S := FDataLink.DataSet.FieldByName(FDataLink.FieldName).AsString;
             ItemIndex := Items.IndexOf(S)
            End
        Else
            Begin
             ItemIndex := IndexOf(FDataLink.Dataset.Bookmark);
            End;
      End;
End;


Procedure TKADaoDBColumnCheckListBox.DataChange(Sender: TObject);
Begin
  If  (Assigned(FDataLink.DataSource))
  And (FDataLink.Active)
  And (FDataLink.FieldName<> '')
  And (NOT FInGetting)
  And (Assigned(FDataLink.Field))
  And (NOT FDataLink.Field.IsBlob)
  And (FDataLink.Dataset.State = dsBrowse) Then
      Begin
       if FGetData Then FillData;
       FGetData := False;
       Resync;
      End;

  If  (Assigned(FDataLink.DataSource))
  And (FDataLink.Active)
  And (FDataLink.FieldName<> '')
  And (NOT FInGetting)
  And (Assigned(FDataLink.Field))
  And (NOT FDataLink.Field.IsBlob)
  And (FDataLink.Dataset.State <> dsBrowse) Then
      Begin
        FGetData := True;
      End;
End;

Procedure TKADaoDBColumnCheckListBox.UpdateData(Sender: TObject);
Var
  S : String;
Begin
  If  (Assigned(FDataLink.DataSource))
  And (FDataLink.Active)
  And (FDataLink.FieldName<> '')
  And (NOT FInGetting)
  And (Assigned(FDataLink.Field))
  And (NOT FDataLink.Field.IsBlob)
  And (FDataLink.CanModify) Then
      Begin
       S := Items.Strings[ItemIndex];
       FDataLink.Dataset.FieldByName(FDataLink.FieldName).AsString := S;
      End;
End;

Procedure TKADaoDBColumnCheckListBox.EditingChange(Sender: TObject);
Begin
  FillData;
End;

Function TKADaoDBColumnCheckListBox.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

Procedure TKADaoDBColumnCheckListBox.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
  FillData;
end;

Procedure TKADaoDBColumnCheckListBox.SetUnique(Value: Boolean);
Begin
 FUnique := Value;
 if CsLoading in ComponentState Then Exit;
 FillData;
End;

Procedure TKADaoDBColumnCheckListBox.SetBookmarks(Value : TStringList);
Begin
 //*********************************** Read Only
End;


Function TKADaoDBColumnCheckListBox.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

Procedure TKADaoDBColumnCheckListBox.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
  FillData;
end;


Function TKADaoDBColumnCheckListBox.GetMasterSource: TDataSource;
Begin
 Result := FMasterLink.DataSource;
End;

Procedure TKADaoDBColumnCheckListBox.SetMasterSource(Value: TDataSource);
Begin
 FMasterLink.DataSource := Value;
 if Value <> nil then Value.FreeNotification(Self);
 FillData;
End;

Procedure TKADaoDBColumnCheckListBox.ML_DataChange(Sender: TObject);
Begin
  FillData;
End;

Procedure TKADaoDBColumnCheckListBox.ML_UpdateData(Sender: TObject);
Begin
  FillData;
End;

Procedure TKADaoDBColumnCheckListBox.ML_EditingChange(Sender: TObject);
Begin
  FillData;
End;

Procedure TKADaoDBColumnCheckListBox.ML_ActiveChange(Sender: TObject);
Begin
  FillData;
End;


procedure Register;
begin
  RegisterComponents('KA Dao Controls', [TKADaoDBColumnCheckListBox]);
end;

end.

