unit KA.Data.KADao.DBStringList;

interface

uses
  System.Text,
  Borland.Vcl.SysUtils,
  Borland.Vcl.Classes,
  Borland.Vcl.DB,
  Borland.Vcl.DBCtrls,
  Borland.Vcl.Controls;

type
  TKADaoDBStringList = class(TComponent)
  private
    { Private declarations }
    FDataLink       : TFieldDataLink;
    FAutoPost       : Boolean;
    FReadOnly       : Boolean;
    FInChange       : Boolean;
    FList           : TStringList;
    Procedure          DataChange(Sender: TObject);
    Procedure          UpdateData(Sender: TObject);
    Procedure          EditingChange(Sender: TObject);
    Procedure          ActiveChange(Sender: TObject);
    Procedure          ListChanged(Sender: TObject);
  protected
    { Protected declarations }
    Function    GetDataField: string;
    Procedure   SetDataField(const Value: string);
    Function    GetDataSource: TDataSource;
    Procedure   SetDataSource(Value: TDataSource);
    Procedure   SetList(Value: TStringList);

    Function    BlobToString(Field:TField):String;
    Procedure   StringToBlob(Field:TField;S:String);
    Procedure   Loaded; override;
    Procedure   Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent);override;
    Destructor  Destroy;override;
  published
    { Published declarations }
    Property    AutoPost          : Boolean     Read FAutoPost       Write FAutoPost;
    Property    DataField         : String      Read GetDataField  Write SetDataField;
    Property    DataSource        : TDataSource Read GetDataSource Write SetDataSource;
    Property    DBStringList      : TStringList Read FList           Write SetList;
    Property    ReadOnly          : Boolean     Read FReadOnly       Write FReadOnly;
  end;

procedure Register;

implementation

Constructor TKADaoDBStringList.Create(AOwner: TComponent);
Begin
  inherited Create(AOwner);
  FAutoPost                 := False;
  FReadOnly                 := False;
  FInChange                 := False;
  FDataLink                 := TFieldDataLink.Create;
  FList                     := TStringList.Create;
  FList.OnChange            := ListChanged;
  FDataLink.Control         := Self;
  FDataLink.OnDataChange    := DataChange;
  FDataLink.OnEditingChange := EditingChange;
  FDataLink.OnUpdateData    := UpdateData;
  FDataLink.OnActiveChange  := ActiveChange;
End;

Destructor TKADaoDBStringList.Destroy;
Begin
 FList.Free;
 FDataLink.Free;
 FDataLink := nil;
 inherited Destroy;
End;

procedure TKADaoDBStringList.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and (AComponent = DataSource) then
      Begin
        DataSource := nil;
      End;
end;

Procedure TKADaoDBStringList.Loaded;
Begin
 Inherited Loaded;
 FList.Clear;
End;

Function TKADaoDBStringList.BlobToString(Field:TField):String;
Var
  MS : TMemoryStream;
Begin
  Result := '';
  MS := TMemoryStream.Create;
  Try
    (Field AS TBlobField).SaveToStream(MS);
    Result := AnsiEncoding.GetString(MS.Memory);
  Finally
    MS.Free;
  End;
End;

Procedure TKADaoDBStringList.StringToBlob(Field:TField;S:String);
Var
  MS : TMemoryStream;
  BA : TBytes;
Begin
  MS := TMemoryStream.Create;
  Try
    BA := AnsiEncoding.GetBytes(S);
    MS.Write(BA,Length(BA));
    MS.Position:=0;
    (FDataLink.Field As TBlobField).LoadFromStream(MS);
  Finally
    MS.Free;
  End;
End;

Procedure TKADaoDBStringList.ListChanged(Sender: TObject);
Begin
 if  FInChange Then Exit;
 If  (Assigned(FDataLink.DataSource))
  And (FDataLink.Active)
  And (FDataLink.FieldName<> '')
  And (Assigned(FDataLink.Field))
  And (FDataLink.Field.IsBlob)
  And (NOT FReadOnly)
  And (FDataLink.CanModify)
  And (FDataLink.Edit)  Then
      Begin
       FDataLink.Modified;
       FDataLink.UpdateRecord;
       if FAutoPost Then FDataLink.DataSet.Post;
      End;
End;


Procedure TKADaoDBStringList.ActiveChange(Sender: TObject);
Begin
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet.Active) Then
      Begin

      End
  Else
      Begin
        FList.Clear;
      End;
End;

Procedure TKADaoDBStringList.DataChange(Sender: TObject);
Var
 MS : TMemoryStream;
Begin
  If  (Assigned(FDataLink.DataSource))
  And (FDataLink.Active)
  And (FDataLink.FieldName<> '')
  And (Assigned(FDataLink.Field))
  And (FDataLink.Field.IsBlob)
  And (FDataLink.Dataset.State = dsBrowse) Then
      Begin
        FInChange := True;
        FList.Clear;
        MS := TMemoryStream.Create;
        Try
         MS.Position:=0;
         (FDataLink.Field AS TBlobField).SaveToStream(MS);
         MS.Position:=0;
         FList.LoadFromStream(MS);
        Finally
         MS.Free;
         FInChange := False;
        End;
      End;
End;

Procedure TKADaoDBStringList.UpdateData(Sender: TObject);
Var
  MS : TMemoryStream;
Begin
  If  (Assigned(FDataLink.DataSource))
  And (FDataLink.Active)
  And (FDataLink.FieldName<> '')
  And (Assigned(FDataLink.Field))
  And (FDataLink.Field.IsBlob)
  And (FDataLink.CanModify) Then
      Begin
        MS := TMemoryStream.Create;
        Try
         MS.Position:=0;
         FList.SaveToStream(MS);
         MS.Position:=0;
         (FDataLink.Field AS TBlobField).LoadFromStream(MS);
        Finally
         MS.Free;
        End;
      End;
End;

Procedure TKADaoDBStringList.EditingChange(Sender: TObject);
Begin

End;

Procedure TKADaoDBStringList.SetList(Value: TStringList);
Begin
  FList.BeginUpdate;
  FList.Assign(Value);
  FList.EndUpdate;
  If  (Assigned(FDataLink.DataSource))
  And (FDataLink.Active)
  And (FDataLink.FieldName<> '')
  And (Assigned(FDataLink.Field))
  And (FDataLink.Field.IsBlob)
  And (NOT FReadOnly)
  And (FDataLink.CanModify)
  And (FDataLink.Edit)  Then
      Begin
       FDataLink.Modified;
       FDataLink.UpdateRecord;
       if FAutoPost Then FDataLink.DataSet.Post;
      End;
End;

Function TKADaoDBStringList.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

Procedure TKADaoDBStringList.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

Function TKADaoDBStringList.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

Procedure TKADaoDBStringList.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

procedure Register;
begin
  RegisterComponents('KA Dao Controls', [TKADaoDBStringList]);
end;

end.
