unit KA.Data.KADao.DateTime;

interface

uses
  System.Text,
  Borland.Vcl.Windows,
  Borland.Vcl.SysUtils,
  Borland.Vcl.Classes,
  Borland.Vcl.Controls,
  Borland.Vcl.DateUtils,
  Borland.Vcl.Dialogs;

type
  TKADaoDateTime = class(TComponent)
  private
    { Private declarations }
    FDateSeparator   : String;
    FShortDateFormat : String;
    FLongDateFormat  : String;
    FTimeSeparator   : String;
    FTimeAMString    : String;
    FTimePMString    : String;
    FShortTimeFormat : String;
    FLongTimeFormat  : String;
    FTimeFormat      : String;
    FShortMonthNames : Array[1..12] of String;
    FLongMonthNames  : Array[1..12] of String;
    FShortDayNames   : Array[1..7]  of String;
    FLongDayNames    : Array[1..7]  of String;
    FHasLongMonth    : Boolean;
    FHasShortMonth   : Boolean;
    FHasDate         : Boolean;
    FHasTime         : Boolean;
    FDen             : Integer;
    FDenOtSedmicata  : Integer;
    FMesec           : Integer;
    FGodina          : Integer;
    FChasove         : Integer;
    FMinuti          : Integer;
    FSecundi         : Integer;
    FMiliSecundi     : Integer;
    FSDOrder         : Integer;
    FLDOrder         : Integer;
    FDateOrder       : Integer;
    FTimeOrder       : Integer;
    FCodePage        : Integer;
    FDate            : TDateTime;
    FTime            : TDateTime;
    FDateTime        : TDateTime;
    FDigits          : TStringList;
    FDateTimeString  : String;
    FSQLString       : String;
    FSQLField        : String;
    FOver2000        : Integer;

    Procedure         SetCodePage(Value : Integer);
    Procedure         SetDate(Value : TDateTime);
    Procedure         SetDateTime(Value : TDateTime);
    Procedure         SetDen(Value : Integer);
    Procedure         SetMesec(Value : Integer);
    Procedure         SetGodina(Value : Integer);
    Procedure         SetChasove(Value : Integer);
    Procedure         SetMinuti(Value : Integer);
    Procedure         SetMiliSecundi(Value : Integer);
    Procedure         SetSecundi(Value : Integer);
    Procedure         SetDenOtSedmicata(Value : Integer);
    Procedure         SetSQLString(Value : String);
    Procedure         SetSQLField(Value : String);
    Procedure         SetDateTimeString(Value : String);

    Procedure         DTError(Orig:String);
    Function          GetLongMonth(Var DTS : String): Integer;
    Function          GetShortMonth(Var DTS : String): Integer;
    Function          RemoveLongDayName(Var DTS : String):Integer;
    Function          RemoveShortDayName(Var DTS : String):Integer;
    Procedure         RemoveNonNumericChars(Var DTS : String);
    Procedure         RemoveExtraSpaces(Var DTS:String);
    Procedure         RemoveFirstSpaces(Var DTS:String);
    Procedure         RemoveLastSpaces(Var DTS:String);
    Procedure         RemoveQuotedData(Var FMTS : String);
    Function          GetLocaleString(Locale, LocaleType: Integer; const Default: String): String;
    Procedure         GetDateAndTime(Const DTS : String;Var Data : String; Var Vreme  : String);
    Procedure         RefreshDataTimeInfo;
    Function          ProcessDateTimeString(Var DTS:String):TDateTime;
  protected
    { Protected declarations }
    procedure         Loaded; override;
  public
    { Public declarations }
    FAccessShorting  : Boolean;
    FBulShort        : Boolean;
    Constructor       Create(AOwner: TComponent);override;
    Destructor        Destroy;override;
  published
    { Published declarations }
    Property AccessShorting : Boolean       Read FAccessShorting Write FAccessShorting;
    Property CodePage       : Integer       Read FCodePage       Write SetCodePage;
    Property Date           : TDateTime     Read FDate           Write SetDate;
    Property DateTime       : TDateTime     Read FDateTime       Write SetDateTime;
    Property Day            : Integer       Read FDen            Write SetDen;
    Property Month          : Integer       Read FMesec          Write SetMesec;
    Property Year           : Integer       Read FGodina         Write SetGodina;
    Property Hours          : Integer       Read FChasove        Write SetChasove;
    Property Minutes        : Integer       Read FMinuti         Write SetMinuti;
    Property MilliSeconds   : Integer       Read FMiliSecundi    Write SetMiliSecundi;
    Property Seconds        : Integer       Read FSecundi        Write SetSecundi;
    Property WeekDay        : Integer       Read FDenOtSedmicata Write SetDenOtSedmicata;
    Property SQLString      : String        Read FSQLString      Write SetSQLString;
    Property SQLField       : String        Read FSQLField       Write SetSQLField;
    Property Over2000       : Integer       Read FOver2000       Write FOver2000;
    Property DateTimeString : String        Read FDateTimeString Write SetDateTimeString;
  end;

procedure Register;

implementation

Constructor TKADaoDateTime.Create(AOwner: TComponent);
Begin
  inherited Create(AOwner);
  FDigits           := TStringList.Create;
  FAccessShorting   := False;
  FBulShort         := False;
  FDen              := 0;
  FMesec            := 0;
  FGodina           := 0;
  FChasove          := 0;
  FMinuti           := 0;
  FSecundi          := 0;
  FMiliSecundi      := 0;
  FDenOtSedmicata   := 0;
  FOver2000         := 10;
  FHasLongMonth     := False;
  FHasShortMonth    := False;
  FHasDate          := False;
  FHasTime          := False;
  FBulShort         := False;
  FSDOrder          := 0;
  FLDOrder          := 0;
  FDateOrder        := 0;
  FTimeOrder        := 0;
  FCodePage         := 0;
  FDate             := 0;
  FTime             := 0;
  FDateTime         := 0;
  FDateTimeString   := '';
  FSQLString        := '';
  FSQLField         := '';
  RefreshDataTimeInfo;
End;

Destructor TKADaoDateTime.Destroy;
Begin
  FDigits.Free;
  Inherited Destroy;
End;

procedure TKADaoDateTime.Loaded;
Begin
  Inherited Loaded;
  if FDateTimeString <> '' Then DateTimeString := FDateTimeString;
End;

Procedure TKADaoDateTime.SetCodePage(Value : Integer);
Begin
 //***************************** Read Only
End;

Procedure TKADaoDateTime.SetDate(Value : TDateTime);
Begin
  FDate           := Value;
  FSQLString      := FormatDateTime('mm"/"dd"/"yyyy', FDate);
  FSQLField       := '#'+FSQLString+'#';
  FDateTimeString := DateToStr(FDate);
  FDenOtSedmicata := DayOfTheWeek(FDate);
End;

Procedure TKADaoDateTime.SetDateTime(Value : TDateTime);
Begin
 FDateTime       := Value;
 FSQLString      := FormatDateTime('mm"/"dd"/"yyyy hh":"nn":"ss', FDateTime);
 FSQLField       := '#'+FSQLString+'#';
 FDateTimeString := DateTimeToStr(FDateTime);
 FDenOtSedmicata :=DayOfWeek(FDateTime);
End;

Procedure TKADaoDateTime.SetDen(Value : Integer);
Begin
 //***************************** Read Only
End;

Procedure TKADaoDateTime.SetMesec(Value : Integer);
Begin
 //***************************** Read Only
End;

Procedure TKADaoDateTime.SetGodina(Value : Integer);
Begin
 //***************************** Read Only
End;

Procedure TKADaoDateTime.SetChasove(Value : Integer);
Begin
 //***************************** Read Only
End;

Procedure TKADaoDateTime.SetMinuti(Value : Integer);
Begin
 //***************************** Read Only
End;

Procedure TKADaoDateTime.SetMiliSecundi(Value : Integer);
Begin
 //***************************** Read Only
End;

Procedure TKADaoDateTime.SetSecundi(Value : Integer);
Begin
 //***************************** Read Only
End;

Procedure TKADaoDateTime.SetDenOtSedmicata(Value : Integer);
Begin
 //***************************** Read Only
End;

Procedure TKADaoDateTime.SetSQLString(Value : String);
Begin
End;

Procedure TKADaoDateTime.SetSQLField(Value : String);
Begin
End;


Procedure TKADaoDateTime.SetDateTimeString(Value : String);
Var
  S : String;
Begin
  FDateTimeString := Value;
  if csLoading in ComponentState Then Exit;
  S := FDateTimeString;
  ProcessDateTimeString(S);
  if FHasDate And  FHasTime Then FSQLString := FormatDateTime('mm"/"dd"/"yyyy hh":"nn":"ss', FDateTime)
  Else
  if FHasDate Then FSQLString := FormatDateTime('mm"/"dd"/"yyyy', FDate)
  Else
  if FHasTime Then FSQLString := FormatDateTime('hh":"nn":"ss', FTime);
  FSQLField := '#'+FSQLString+'#';
End;

Procedure TKADaoDateTime.DTError(Orig:String);
Var
 S : String;
Begin
 S := 'Invalid format!';
 if FHasDate And FHasTime Then S := Orig+' is not a valid date and time.'
 Else
 if FHasDate               Then S := Orig+' is not a valid date.'
 Else
 if FHasTime               Then S := Orig+' is not a valid time.';
 Raise EConvertError.Create(S);
End;

Function TKADaoDateTime.GetLongMonth(Var DTS : String): Integer;          
Var
   X : Integer;
   P : Integer;
Begin
   Result := 0;
   For X := 1 to 12 do
      Begin
        P := Pos(WideLowerCase(FLongMonthNames[X]),WideLowerCase(DTS));
        if P > 0 Then
           Begin
             Result := X;
             Delete(DTS,P,Length(FLongMonthNames[X]));
             Insert(' ',DTS,P);
             Exit;
           End;
      End;
End;

Function TKADaoDateTime.GetShortMonth(Var DTS : String): Integer;
Var
   X : Integer;
   P : Integer;
Begin
   Result := 0;
   For X := 1 to 12 do
      Begin
        P := Pos(WideLowerCase(FShortMonthNames[X]),WideLowerCase(DTS));
        if P > 0 Then
           Begin
             Result := X;
             Delete(DTS,P,Length(FShortMonthNames[X]));
             Insert(' ',DTS,P);
             Exit;
           End;
      End;
End;

Function TKADaoDateTime.RemoveLongDayName(Var DTS : String):Integer;
Var
   X : Integer;
   P : Integer;
Begin
  Result := 0;
  For X := 1 to 7 do
      Begin
        P := Pos(WideLowerCase(FLongDayNames[X]),WideLowerCase(DTS));
        if P > 0 Then
           Begin
             Result := X;
             Delete(DTS,P,Length(FLongDayNames[X]));
             Insert(' ',DTS,P);
             Exit;
           End;
      End;
End;

Function TKADaoDateTime.RemoveShortDayName(Var DTS : String):Integer;
Var
   X : Integer;
   P : Integer;
Begin
  Result := 0;
  For X := 1 to 7 do
      Begin
        P := Pos(WideLowerCase(FShortDayNames[X]),WideLowerCase(DTS));
        if P > 0 Then
           Begin
             Result := X;
             Delete(DTS,P,Length(FShortDayNames[X]));
             Insert(' ',DTS,P);
             Exit;
           End;
      End;
End;

Procedure TKADaoDateTime.RemoveNonNumericChars(Var DTS : String);
Var
 CH : Char;
 X  : Integer;
 L  : Integer;
Begin
 L  := Length(DTS);
 if L=0 Then Exit;
 For X := 1 to L do
   Begin
     CH := DTS[X];
     if (NOT IsCharAlpha(CH)) And (IsCharAlphaNumeric(CH)) Then
        Begin
        End
     Else
        Begin
         DTS[X]:=' ';
        End;
   End;
End;

Procedure TKADaoDateTime.RemoveExtraSpaces(Var DTS:String);
Var
 P : Integer;
Begin
 P := Pos('  ',DTS);
 While P > 0 do
   Begin
    Delete(DTS,P,1);
    P := Pos('  ',DTS);
   End;
End;

Procedure TKADaoDateTime.RemoveFirstSpaces(Var DTS:String);
Begin
 While (Length(DTS) > 0) And (DTS[1]=' ') do
   Begin
    Delete(DTS,1,1);
   End;
End;

Procedure TKADaoDateTime.RemoveLastSpaces(Var DTS:String);
Var
 L : Integer;
Begin
 L := Length(DTS);
 While (L > 0) And (DTS[L]=' ') do
   Begin
    Delete(DTS,L,1);
    L := Length(DTS);
   End;
End;

Procedure TKADaoDateTime.RemoveQuotedData(Var FMTS : String);
Var
  X    : Integer;
  L    : Integer;
  FIQ  : Boolean;
Begin
  L := Length(FMTS);
  if L=0 Then Exit;
  FIQ := False;
  For X := 1 to L do
    Begin
     if FMTS[X]='''' Then
        Begin
          FMTS[X] := ' ';
          FIQ := NOT FIQ;
        End;
     if FIQ THEN FMTS[X] := ' ';
    End;
  RemoveExtraSpaces(FMTS);
  RemoveFirstSpaces(FMTS);
  RemoveLastSpaces (FMTS);
End;

Function TKADaoDateTime.GetLocaleString(Locale, LocaleType: Integer; const Default: String): String;
var
  L      : Integer;
  Buffer : StringBuilder;
begin
  Buffer := StringBuilder.Create(1000);
  L := GetLocaleInfo(Locale, LocaleType, Buffer, 1000);
  if L > 0 then
     Result := Buffer.ToString
  else
     Result := Default;
  Buffer.Free;
end;

Procedure TKADaoDateTime.RefreshDataTimeInfo;
Var
  X      : Integer;
Begin
  FDateSeparator   := DateSeparator;
  FShortDateFormat := ShortDateFormat;
  FLongDateFormat  := LongDateFormat;
  FTimeSeparator   := TimeSeparator;
  FTimeAMString    := TimeAMString;
  FTimePMString    := TimePMString;
  FShortTimeFormat := ShortTimeFormat;
  FLongTimeFormat  := LongTimeFormat;
  FTimeFormat      := GetLocaleString(GetThreadLocale, LOCALE_STIMEFORMAT, 'hh:mm:ss');
  FSDOrder         := StrToInt(GetLocaleString(GetThreadLocale, LOCALE_IDATE,  '0'));
  FLDOrder         := StrToInt(GetLocaleString(GetThreadLocale, LOCALE_ILDATE, '0'));
  FTimeOrder       := StrToInt(GetLocaleString(GetThreadLocale, LOCALE_ITIME, '0'));
  For X := 1 to 12 do
      Begin
       FShortMonthNames[X] := ShortMonthNames[X];
       FLongMonthNames[X]  := LongMonthNames[X];
      End;
  For X := 1 to 7 do
      Begin
       FShortDayNames[X]   := ShortDayNames[X];
       FLongDayNames[X]    := LongDayNames[X];
      End;
  FCodePage := GetACP;
  if FCodePage=1251 Then
     Begin
       if WideLowerCase(FShortDayNames[5])='' then ShortDayNames[5][6]:='';
       if WideLowerCase(FLongDayNames[5]) ='' then LongDayNames[5][6]:='';
       FShortDayNames[5]   := ShortDayNames[5];
       FLongDayNames[5]    := LongDayNames[5];
       If FBulShort Then
          Begin
           FShortDayNames[1]    := '.';
           FShortDayNames[2]    := '.';
           FShortDayNames[3]    := '.';
           FShortDayNames[4]    := '.';
           FShortDayNames[5]    := '.';
           FShortDayNames[6]    := '.';
           FShortDayNames[7]    := '.';
           //*******************************************************************
          End;
     End;
  if FAccessShorting Then
     Begin
       //*******************************************************************
       FShortMonthNames[1]  := Copy(FShortMonthNames[1],1,7);
       FShortMonthNames[2]  := Copy(FShortMonthNames[2],1,7);
       FShortMonthNames[3]  := Copy(FShortMonthNames[3],1,7);
       FShortMonthNames[4]  := Copy(FShortMonthNames[4],1,7);
       FShortMonthNames[5]  := Copy(FShortMonthNames[5],1,7);
       FShortMonthNames[6]  := Copy(FShortMonthNames[6],1,7);
       FShortMonthNames[7]  := Copy(FShortMonthNames[7],1,7);
       FShortMonthNames[8]  := Copy(FShortMonthNames[8],1,7);
       FShortMonthNames[9]  := Copy(FShortMonthNames[9],1,7);
       FShortMonthNames[10] := Copy(FShortMonthNames[10],1,7);
       FShortMonthNames[11] := Copy(FShortMonthNames[11],1,7);
       FShortMonthNames[12] := Copy(FShortMonthNames[12],1,7);
       //*******************************************************************
     End;
End;

Procedure TKADaoDateTime.GetDateAndTime(Const DTS : String;Var Data : String; Var Vreme  : String);
Var
 PDD : Integer;
 PTD : Integer;
 X   : Integer;
 CH  : Char;
 S   : String;
Begin
 PDD   := Pos(FDateSeparator,DTS);
 PTD   := Pos(FTimeSeparator,DTS);
 Data  := DTS;
 Vreme := '';
 S     := DTS;
 if PTD = 0 Then Exit;
 If PTD > PDD Then
    Begin
     For X := 1 to PTD-1 do
         Begin
          CH := S[X];
          if (NOT IsCharAlpha(CH)) And (IsCharAlphaNumeric(CH)) Then
             Begin
             End
          Else
             Begin
              S[X]:=' ';
             End;
         End;
     Repeat
       Dec(PTD);
     Until (PTD=0) or (DTS[PTD]=' ');
     Data  := Copy(S,1,PTD);
     Delete(S,1,PTD);
     Vreme := S;
    End;
End;

Function  TKADaoDateTime.ProcessDateTimeString(Var DTS:String):TDateTime;
Var
  Orig      : String;
  Data      : String;
  Vreme     : String;
  L         : Integer;
  DD,MM,YY  : Word;
Begin
  Result          := 0;
  if DTS           = '' Then Exit;
  //****************************************************************************
  Orig            := DTS;
  FDen           := 0;
  FMesec         := 0;
  FGodina        := 0;
  FChasove       := 0;
  FMinuti        := 0;
  FSecundi       := 0;
  FMiliSecundi   := 0;
  FHasLongMonth  := False;
  FHasShortMonth := False;
  FHasDate       := False;
  FHasTime       := False;
  //****************************************************************************
  RefreshDataTimeInfo;
  //****************************************************************************
  FMesec := GetLongMonth(DTS);
  if FMesec > 0 Then FHasLongMonth := True;
  if FMesec = 0 Then
     Begin
       FMesec := GetShortMonth(DTS);
       if FMesec > 0 Then FHasShortMonth := True;
     End;
  FDenOtSedmicata := RemoveLongDayName(DTS);
  if FDenOtSedmicata=0 Then FDenOtSedmicata := RemoveShortDayName(DTS);
  //****************************************************************************
  FHasDate := (Pos(FDateSeparator,DTS) > 0);
  if FMesec > 0 Then FHasDate := True;
  FHasTime := (Pos(FTimeSeparator,DTS) > 0);
  Data   := '';
  Vreme  := '';
  GetDateAndTime(DTS,Data,Vreme);
  RemoveNonNumericChars(Data);
  RemoveExtraSpaces(Data);
  RemoveFirstSpaces(Data);
  RemoveLastSpaces(Data);
  If Length(Data) > 0 Then FHasDate := True;
  //****************************************************************************
  Data   := '';
  Vreme  := '';
  if FHasDate And FHasTime Then
     Begin
      Data   := '';
      Vreme  := '';
      GetDateAndTime(DTS,Data,Vreme);
     End
  Else
  if FHasDate Then
     Begin
      Data   := DTS;
      Vreme  := '';
     End
  Else
  if FHasTime Then
     Begin
      Data   := '';
      Vreme  := DTS;
     End
  Else
  DTError(Orig);

  RemoveNonNumericChars(Data);
  RemoveExtraSpaces(Data);
  RemoveFirstSpaces(Data);
  RemoveLastSpaces(Data);

  RemoveNonNumericChars(Vreme);
  RemoveExtraSpaces(Vreme);
  RemoveFirstSpaces(Vreme);
  RemoveLastSpaces(Vreme);

  if FHasDate Then
     Begin
       FDigits.Clear;
       if FHasLongMonth Then FDateOrder := FLDOrder Else FDateOrder := FSDOrder;
       L := Length(Data);
       if L = 0 Then DTError(Orig);
       Data := StringReplace(Data,' ',#13+#10,[rfReplaceAll]);
       FDigits.Text := Data;
       if FDigits.Count < 2 Then DTError(Orig);
       Case FDateOrder of
            0 : Begin {mdy}
                  if FMesec=0 Then
                     Begin
                       if FDigits.Count < 3 Then DTError(Orig);
                       FMesec  := StrToInt(FDigits.Strings[0]);
                       FDen    := StrToInt(FDigits.Strings[1]);
                     End
                  Else
                     Begin
                       FDen    := StrToInt(FDigits.Strings[1]);
                     End;
                  FGodina      := StrToInt(FDigits.Strings[2]);
                End;
            1 : Begin {dmy}
                  FDen := StrToInt(FDigits.Strings[0]);
                  if FMesec=0 Then
                     Begin
                       if FDigits.Count < 3 Then DTError(Orig);
                       FMesec  := StrToInt(FDigits.Strings[1]);
                       FGodina := StrToInt(FDigits.Strings[2]);
                     End
                  Else
                     Begin
                       FGodina := StrToInt(FDigits.Strings[1]);
                     End;
                End;
            2 : Begin {ymd}
                  FGodina := StrToInt(FDigits.Strings[0]);
                  if FMesec=0 Then
                     Begin
                       if FDigits.Count < 3 Then DTError(Orig);
                       FMesec  := StrToInt(FDigits.Strings[1]);
                       FDen    := StrToInt(FDigits.Strings[2]);
                     End
                  Else
                     Begin
                       FDen    := StrToInt(FDigits.Strings[1]);
                     End;
                End;
       End;
     End;
  //****************************************************************************
  if FHasTime Then
     Begin
      FDigits.Clear;
      L := Length(Vreme);
      if L = 0 Then DTError(Orig);
      Vreme := StringReplace(Vreme,' ',#13+#10,[rfReplaceAll]);
      FDigits.Text := Vreme;
      if FDigits.Count < 2 Then DTError(Orig);
      Case FTimeOrder of
          0 : Begin
                FChasove     := StrToInt(FDigits.Strings[0]);
                FMinuti      := StrToInt(FDigits.Strings[1]);
                if FDigits.Count > 2 Then FSecundi     := StrToInt(FDigits.Strings[2]);
                if FDigits.Count > 3 Then FMiliSecundi := StrToInt(FDigits.Strings[3]);
              End;
          1 : Begin
                FChasove     := StrToInt(FDigits.Strings[0]);
                FMinuti      := StrToInt(FDigits.Strings[1]);
                if FDigits.Count > 2 Then FSecundi     := StrToInt(FDigits.Strings[2]);
                if FDigits.Count > 3 Then FMiliSecundi := StrToInt(FDigits.Strings[3]);
              End;
      End;
     End;
  //****************************************************************************
  FDate      := 0;
  FTime      := 0;
  FDateTime  := 0;
  if FHasDate And FHasTime Then
     Begin
      if FGodina < FOver2000+1 Then FGodina := FGodina+2000;
      if (FGodina > FOver2000) And (FGodina < 99) Then FGodina := FGodina+1900;
      FDate      := EncodeDate(FGodina, FMesec, FDen);
      FTime      := EncodeTime(FChasove, FMinuti, FSecundi, FMiliSecundi);
      FDateTime  := FDate+FTime;
      FTime      := FDateTime;
      Result      := FDateTime;
      if FDenOtSedmicata=0 Then FDenOtSedmicata:=DayOfTheWeek(Result);
     End
  Else
  if FHasDate Then
     Begin
      if FGodina < FOver2000+1 Then FGodina := FGodina+2000;
      if (FGodina > FOver2000) And (FGodina < 99) Then FGodina := FGodina+1900;
      FDate      := EncodeDate(FGodina, FMesec, FDen);
      FDateTime  := FDate;
      Result      := FDate;
      if FDenOtSedmicata=0 Then FDenOtSedmicata:=DayOfTheWeek(Result);
     End
  Else
  if FHasTime Then
     Begin
       DecodeDate(Now, YY, MM, DD);
       FDen      := DD;
       FMesec    := MM;
       FGodina   := YY;
       FTime     := EncodeDate(FGodina, FMesec, FDen)+EncodeTime(FChasove, FMinuti, FSecundi, FMiliSecundi);
       FDateTime := FTime;
       Result     := FTime;
     End;
  if FDenOtSedmicata > 0 Then
     Begin
       if DayOfTheWeek(Result) <> FDenOtSedmicata Then DTError(Orig);
     End;
  //****************************************************************************
End;

procedure Register;
begin
  RegisterComponents('KA Dao Controls', [TKADaoDateTime]);
end;

end.
