unit KA.Data.KADao.ExportButton;

interface

uses
  Borland.Vcl.SysUtils,
  Borland.Vcl.Classes,
  Borland.Vcl.Controls,
  Borland.Vcl.Forms,
  Borland.Vcl.Dialogs,
  Borland.Vcl.StdCtrls,
  Borland.Vcl.DB,
  Borland.Vcl.DBCtrls;

type
  TKADaoExportButton = class(TButton)
  private
    { Private declarations }
    FDataLink     : TFieldDataLink;
    FBefore       : Boolean;
    FExportDir    : String;
  protected
    { Protected declarations }
    Function    GetDataSource: TDataSource;
    Procedure   SetDataSource(Value: TDataSource);
    Procedure   Notification(AComponent: TComponent; Operation: TOperation); override;
    Procedure   HandleClickEvent;
    Procedure   DataChange(Sender: TObject);
    Procedure   ActiveChange(Sender: TObject);
    procedure   Loaded; override;
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent);override;
    Destructor  Destroy;override;
    Procedure   Click; override;
  published
    { Published declarations }
    Property    DataSource       : TDataSource Read GetDataSource Write SetDataSource;
    Property    CallBefore       : Boolean     Read FBefore         Write FBefore;
    Property    DefaultExportDir : String      Read FExportDir      Write FExportDir;
  end;

procedure Register;

implementation
Uses
  KA.Data.KADao.Table,
  KA.Data.KADao.SASUnit;

Constructor TKADaoExportButton.Create(AOwner: TComponent);
Begin
  inherited Create(AOwner);
  Caption     := 'Export';
  FExportDir := '';
  FDataLink  := TFieldDataLink.Create;
  FDataLink.OnDataChange    := DataChange;
  FDataLink.OnActiveChange  := ActiveChange;
  FBefore    := True;
End;

Destructor TKADaoExportButton.Destroy;
Begin
  FDataLink.Free;
  FDataLink := nil;
  Inherited Destroy;
End;

Function TKADaoExportButton.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

Procedure TKADaoExportButton.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (NOT DataSource.DataSet.ISEmpty)
  And (DataSource.DataSet.Active) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
end;

Procedure TKADaoExportButton.ActiveChange(Sender: TObject);
Begin
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (NOT DataSource.DataSet.ISEmpty)
  And (DataSource.DataSet.Active) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

Procedure TKADaoExportButton.DataChange(Sender: TObject);
Begin
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (NOT DataSource.DataSet.ISEmpty)
  And (DataSource.DataSet.Active) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

Procedure TKADaoExportButton.Loaded;
Begin
  Inherited Loaded;
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (NOT DataSource.DataSet.ISEmpty)
  And (DataSource.DataSet.Active) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

procedure TKADaoExportButton.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and (AComponent = DataSource) then DataSource := nil;
end;

Procedure TKADaoExportButton.HandleClickEvent;
Begin
 if  Assigned(DataSource)
 And Assigned(DataSource.Dataset)
 And (DataSource.DataSet is TKaDaoTable)
 And (NOT DataSource.DataSet.ISEmpty)
 And (DataSource.DataSet.Active) Then
     Begin
      Application.CreateForm(TSAS, SAS);
      SAS.ExportDir:=FExportDir;
      if SAS.Execute(DataSource.DataSet) Then
          Begin
          End;
       SAS.Free;
     End;
End;

Procedure TKADaoExportButton.Click;
Begin
 if FBefore Then inherited Click;
 HandleClickEvent;
 if NOT FBefore Then inherited Click;
End;

procedure Register;
begin
  RegisterComponents('KA Dao Controls', [TKADaoExportButton]);
end;

end.
